﻿Imports MVVM.Model
Imports MVVM.View
Imports System.Collections.ObjectModel
Imports System.ComponentModel

Namespace ViewModel

    Public Class MainViewModel
        Inherits ViewModelBase
#Region "Konstruktor"

        Private Sub New()
            DataPfad = My.Settings.DB_Pfad
            Me.Nummer = "OneInstanz " & Counter.ToString
            Debug.Print("Private New")
            Reload()
            'Init()
        End Sub

        'Singleton Instanz
        Private Shared _OneInstanz As MainViewModel
        Public Shared ReadOnly Property OneInstanz As MainViewModel
            Get
                If _OneInstanz Is Nothing Then
                    _OneInstanz = New MainViewModel
                End If
                Return _OneInstanz
            End Get
        End Property

#End Region

#Region "Datenmodel holen"
        'Daten von Model laden
        Public Shared DataPfad As String

        Private _data As MyDBDataContext
        Public Sub Reload()
            If _data IsNot Nothing Then _data.Dispose()

            _data = New MyDBDataContext("Data Source=" & DataPfad)
            If _data.DatabaseExists() Then
                _data.Log = Console.Out

                Me.PersonenListCollection = New ListCollectionView(_data.Person.GetList) With {.Filter = Function(o) True}
                OnPropertychanged("PersonenListCollection")

            Else
                _data.CreateDatabase()
            End If

        End Sub

        Public Property PersonenListCollection As New ListCollectionView({New Person("dummy", "Mock")}) With {.Filter = Function(o) True}
        Public Property ColorListCollection As New ListCollectionView({New Farbe With {.Wert = Colors.Yellow.ToString}}) With {.Filter = Function(o) True}
#End Region


#Region "Workspaces"
        Private _workspaces As ObservableCollection(Of WorkspaceViewModel)
        Public ReadOnly Property WorkSpaces As ObservableCollection(Of WorkspaceViewModel)
            Get
                If _workspaces Is Nothing Then
                    _workspaces = New ObservableCollection(Of WorkspaceViewModel)
                    AddHandler _workspaces.CollectionChanged, AddressOf OnWorkspacesChanged
                End If
                Return _workspaces
            End Get
        End Property

        Private Sub OnWorkspacesChanged(sender As Object, e As Specialized.NotifyCollectionChangedEventArgs)
            If e.NewItems IsNot Nothing AndAlso e.NewItems.Count <> 0 Then
                For Each workspace As WorkspaceViewModel In e.NewItems
                    AddHandler workspace.RequestClose, AddressOf OnWorkspaceRequestClose
                Next workspace
            End If

            If e.OldItems IsNot Nothing AndAlso e.OldItems.Count <> 0 Then
                For Each workspace As WorkspaceViewModel In e.OldItems
                    RemoveHandler workspace.RequestClose, AddressOf OnWorkspaceRequestClose
                Next workspace
            End If
        End Sub

        Private Sub OnWorkspaceRequestClose(sender As Object, e As EventArgs)
            Dim workspace As WorkspaceViewModel = TryCast(sender, WorkspaceViewModel)
            workspace.Dispose()
            Me.WorkSpaces.Remove(workspace)
        End Sub
#End Region

#Region "Commands"
        Private _commands As List(Of CommandViewModel)
        Public ReadOnly Property commands As List(Of CommandViewModel)
            Get
                If _commands Is Nothing Then
                    Dim cmds As List(Of CommandViewModel) = Me.CreateCommands()
                    _commands = New List(Of CommandViewModel)(cmds)
                End If
                Return _commands
            End Get
        End Property

        Function CreateCommands() As List(Of CommandViewModel)
            Dim _commandList As New List(Of CommandViewModel)
            Dim _View1Command = New RelayCommand(AddressOf PersonenListe_ShowExekute, AddressOf Can_PersonenListe_ShowExekute)
            Dim _View2Command = New RelayCommand(AddressOf View2_ShowExekute, AddressOf Can_View2_ShowExekute)
            Dim view1 As New CommandViewModel("Personenliste", _View1Command)
            Dim view2 As New CommandViewModel("ColorView", _View2Command)

            _commandList.Add(view1)
            _commandList.Add(view2)
            Return _commandList
        End Function


#Region "Dateicommands"
        Private _DateiCommands As List(Of CommandViewModel)
        Public ReadOnly Property DateiCommands As List(Of CommandViewModel)
            Get
                If _DateiCommands Is Nothing Then
                    Dim loadCommand As New RelayCommand(AddressOf ReloadExecute)
                    Dim saveCommand As New RelayCommand(AddressOf SaveExecute)
                    Dim ReloadView As New CommandViewModel("Laden", loadCommand)
                    Dim SaveView As New CommandViewModel("Speichern", saveCommand)

                    Dim cmds As New List(Of CommandViewModel)
                    cmds.Add(ReloadView)
                    cmds.Add(SaveView)
                    _DateiCommands = New List(Of CommandViewModel)(cmds)
                End If
                Return _DateiCommands
            End Get
        End Property

        Private Sub ReloadExecute(sender As Object)
            Reload()
            Debug.Print("Laden mit Instanz Nummer: {0}", Me.Nummer)
        End Sub

        Private Sub SaveExecute(sender As Object)
            Try
                Debug.Print("Speichern mit Instanz Nummer: {0}", Me.Nummer)
                Debug.Print("eingefügte Daten: {0}", _data.GetChangeSet.Inserts.Count)
                Debug.Print("gelöschte Daten: {0}", _data.GetChangeSet.Deletes.Count)
                Debug.Print("geänderte Daten: {0}", _data.GetChangeSet.Updates.Count)

                _data.SubmitChanges()
            Catch ex As Exception
                Console.WriteLine(ex.Message)
                MessageBox.Show(ex.Message, "Speichern fehlgeschlagen!")
            End Try
        End Sub
#End Region

#End Region

#Region "Personenliste Show"

        Dim PVM As PersonenViewModel
        Private Sub PersonenListe_ShowExekute(sender As Object)
            'kann 1x geöffnet werden
            PVM = New PersonenViewModel
            PVM.DisplayName = "Personenliste"
            PVM.PersonenListCollection = Me.PersonenListCollection
            WorkSpaces.Add(PVM)
            SetActiveWorkspace(PVM)
        End Sub

        Private Function Can_PersonenListe_ShowExekute(sender As Object) As Boolean
            'erlaubt das Einmalige Öffnen der Ansicht Personenliste
            If WorkSpaces.Contains(PVM) Then
                Return False
            Else
                Return True
            End If
        End Function
#End Region

#Region "ColorView Show"

        Dim View1_Count As Integer
        Private Sub View2_ShowExekute(sender As Object)
            'kann beliebig oft geöffnet werden
            View1_Count += 1
            Dim CV As New ColorViewModel
            CV.DisplayName = "Color " & View1_Count
            WorkSpaces.Add(CV)
            SetActiveWorkspace(CV)
        End Sub

        Private Function Can_View2_ShowExekute(sender As Object) As Boolean
            Return True
        End Function
#End Region

#Region "Helferlein"
        ''' <summary>
        ''' Die Sub setzt den Focus auf das im Parameter übergebene WorkSpaceViewModel
        ''' </summary>
        ''' <param name="workspace">Das zu focussierende WorkSpaceViewModel</param>
        ''' <remarks></remarks>
        Private Sub SetActiveWorkspace(ByVal workspace As WorkspaceViewModel)
            Debug.Assert(Me.WorkSpaces.Contains(workspace)) 'Aufrufliste wenn das angesprochene Viewmodel nicht in der Auflistung existiert

            Dim collectionView As ICollectionView = CollectionViewSource.GetDefaultView(Me.WorkSpaces)
            If collectionView IsNot Nothing Then
                collectionView.MoveCurrentTo(workspace)
            End If

        End Sub
#End Region

    End Class
End Namespace


